@@ -53,6 +53,7 @@
 struct omap3_mmc_s
 {
     SysBusDevice busdev;
+    MemoryRegion iomem;
     qemu_irq irq;
     qemu_irq dma[2];
     qemu_irq coverswitch;
@@ -734,16 +735,20 @@ static void omap3_mmc_write(void *opaque, target_phys_addr_t addr,
     }
 }
 
-static CPUReadMemoryFunc *omap3_mmc_readfn[] = {
-    omap_badwidth_read32,
-    omap_badwidth_read32,
-    omap3_mmc_read,
-};
-
-static CPUWriteMemoryFunc *omap3_mmc_writefn[] = {
-    omap_badwidth_write32,
-    omap_badwidth_write32,
-    omap3_mmc_write,
+static const MemoryRegionOps omap3_mmc_ops = {
+    .old_mmio = {
+        .read = {
+            omap_badwidth_read32,
+            omap_badwidth_read32,
+            omap3_mmc_read,
+        },
+        .write = {
+            omap_badwidth_write32,
+            omap_badwidth_write32,
+            omap3_mmc_write,
+        },
+    },
+    .endianness = DEVICE_NATIVE_ENDIAN,
 };
 
 static int omap3_mmc_init(SysBusDevice *dev)
@@ -752,23 +757,30 @@ static int omap3_mmc_init(SysBusDevice *dev)
     sysbus_init_irq(dev, &s->irq);
     sysbus_init_irq(dev, &s->dma[0]);
     sysbus_init_irq(dev, &s->dma[1]);
-    sysbus_init_mmio(dev, 0x1000,
-                     cpu_register_io_memory(omap3_mmc_readfn,
-                                            omap3_mmc_writefn, s,
-                                            DEVICE_NATIVE_ENDIAN));
+    memory_region_init_io(&s->iomem, &omap3_mmc_ops, s,
+                          "omap3_mmc", 0x1000);
+    sysbus_init_mmio(dev, &s->iomem);
     return 0;
 }
 
-static SysBusDeviceInfo omap3_mmc_info = {
-    .init = omap3_mmc_init,
-    .qdev.name = "omap3_mmc",
-    .qdev.size = sizeof(struct omap3_mmc_s),
-    .qdev.reset = omap3_mmc_reset
+static void omap3_mmc_class_init(ObjectClass *klass, void *data)
+{
+    DeviceClass *dc = DEVICE_CLASS(klass);
+    SysBusDeviceClass *k = SYS_BUS_DEVICE_CLASS(klass);
+    k->init = omap3_mmc_init;
+    dc->reset = omap3_mmc_reset;
+}
+
+static TypeInfo omap3_mmc_info = {
+    .name = "omap3_mmc",
+    .parent = TYPE_SYS_BUS_DEVICE,
+    .instance_size = sizeof(struct omap3_mmc_s),
+    .class_init = omap3_mmc_class_init,
 };
 
-static void omap3_mmc_register_device(void)
+static void omap3_mmc_register_types(void)
 {
-    sysbus_register_withprop(&omap3_mmc_info);
+    type_register_static(&omap3_mmc_info);
 }
 
 void omap3_mmc_attach(DeviceState *dev, BlockDriverState *bs,
@@ -782,4 +794,4 @@ void omap3_mmc_attach(DeviceState *dev, BlockDriverState *bs,
     s->card = sd_init(bs, is_spi, is_mmc);
 }
 
-device_init(omap3_mmc_register_device)
+type_init(omap3_mmc_register_types)
