@@ -45,11 +45,16 @@ static uint64_t omap_tap_read(void *opaque, target_phys_addr_t addr,
         case omap3630:
             return 0x1b89102f;  /* ES 1.1 */
         default:
-            hw_error("%s: Bad mpu model\n", __FUNCTION__);
+            hw_error("%s: Bad mpu model\n", __func__);
         }
 
+        /* For OMAP2 there is a two word PRODUCTION_ID register at
+         * 0x208..0x210.
+         * For OMAP3 there is a four word PRODUCTION_ID register at
+         * 0x208..0x214.
+         */
+
     case 0x208: /* PRODUCTION_ID_reg bits 0-31 */
-    case 0x210: /* PRODUCTION_ID reg bits 64-95 */
         switch (s->mpu_model) {
         case omap2420:
             return 0x000254f0;	/* POP ESHS2.1.1 in N91/93/95, ES2 in N800 */
@@ -61,12 +66,12 @@ static uint64_t omap_tap_read(void *opaque, target_phys_addr_t addr,
             return 0x000000f0;
         case omap3430:
         case omap3630:
-            return 0x000f00f0;
+            return 0x0;
         default:
-            hw_error("%s: Bad mpu model\n", __FUNCTION__);
+            hw_error("%s: Bad mpu model\n", __func__);
         }
+        break;
     case 0x20c: /* PRODUCTION_ID bits 32-63 */
-    case 0x214: /* PRODUCTION_ID bits 96-127 */
         switch (s->mpu_model) {
         case omap2420:
         case omap2422:
@@ -74,13 +79,44 @@ static uint64_t omap_tap_read(void *opaque, target_phys_addr_t addr,
             return 0xcafeb5d9;	/* ES 2.2 */
         case omap2430:
             return 0xcafeb68a;	/* ES 2.2 */
+        case omap3430:
+        case omap3630:
+            return 0x0;
+        default:
+            hw_error("%s: Bad mpu model\n", __func__);
+        }
+        break;
+    case 0x210: /* PRODUCTION_ID reg bits 64-95 (OMAP3 only) */
+        switch (s->mpu_model) {
+        case omap2420:
+        case omap2422:
+        case omap2423:
+        case omap2430:
+            /* OMAP3 only, break out to bad-register path */
+            break;
+        case omap3430:
+        case omap3630:
+            return 0x000f00f0;
+        default:
+            hw_error("%s: Bad mpu model\n", __func__);
+        }
+        break;
+    case 0x214: /* PRODUCTION_ID bits 96-127 (OMAP3 only) */
+        switch (s->mpu_model) {
+        case omap2420:
+        case omap2422:
+        case omap2423:
+        case omap2430:
+            /* OMAP3 only, break out to bad-register path */
+            break;
         case omap3430:
             return 0xcafeb7ae;  /* ES 2.x/3.0 */
         case omap3630:
             return 0xcafeb891;  /* ES 1.0/1.1 */
         default:
-            hw_error("%s: Bad mpu model\n", __FUNCTION__);
+            hw_error("%s: Bad mpu model\n", __func__);
         }
+        break;
 
     case 0x218:	/* DIE_ID_reg */
         return ('Q' << 24) | ('E' << 16) | ('M' << 8) | ('U' << 0);
