@@ -40,6 +40,7 @@ extern CPUWriteMemoryFunc *musb_write[];
 
 typedef struct omap3_hsusb_otg_s {
     SysBusDevice busdev;
+    MemoryRegion iomem;
     qemu_irq mc_irq;
     qemu_irq dma_irq;
     qemu_irq stdby_irq;
@@ -136,7 +137,7 @@ static void omap3_hsusb_otg_write(int access,
     else switch (addr) {
         case 0x400: /* OTG_REVISION */
         case 0x408: /* OTG_SYSSTATUS */
-            OMAP_RO_REGV(addr, value);
+            OMAP_RO_REG(addr);
             break;
         case 0x404: /* OTG_SYSCONFIG */
             TRACE("OTG_SYSCONFIG = 0x%08x", value);
@@ -160,7 +161,7 @@ static void omap3_hsusb_otg_write(int access,
             omap3_hsusb_otg_stdby_update(s);
             break;
         default:
-            OMAP_BAD_REGV(addr, value);
+            OMAP_BAD_REG(addr);
             break;
     }
 }
@@ -198,16 +199,20 @@ static void omap3_hsusb_otg_writew(void *opaque, target_phys_addr_t addr,
     omap3_hsusb_otg_write(2, opaque, addr, value);
 }
 
-static CPUReadMemoryFunc *omap3_hsusb_otg_readfn[] = {
-    omap3_hsusb_otg_readb,
-    omap3_hsusb_otg_readh,
-    omap3_hsusb_otg_readw,
-};
-
-static CPUWriteMemoryFunc *omap3_hsusb_otg_writefn[] = {
-    omap3_hsusb_otg_writeb,
-    omap3_hsusb_otg_writeh,
-    omap3_hsusb_otg_writew,
+static const MemoryRegionOps omap3_hsusb_otg_ops = {
+    .old_mmio = {
+        .read = {
+            omap3_hsusb_otg_readb,
+            omap3_hsusb_otg_readh,
+            omap3_hsusb_otg_readw,
+        },
+        .write = {
+            omap3_hsusb_otg_writeb,
+            omap3_hsusb_otg_writeh,
+            omap3_hsusb_otg_writew,
+        },
+    },
+    .endianness = DEVICE_NATIVE_ENDIAN,
 };
 
 static void omap3_hsusb_musb_core_intr(void *opaque, int source, int level)
@@ -223,25 +228,35 @@ static int omap3_hsusb_otg_init(SysBusDevice *dev)
     sysbus_init_irq(dev, &s->mc_irq);
     sysbus_init_irq(dev, &s->dma_irq);
     sysbus_init_irq(dev, &s->stdby_irq);
-    sysbus_init_mmio(dev, 0x1000,
-                     cpu_register_io_memory(omap3_hsusb_otg_readfn,
-                                            omap3_hsusb_otg_writefn, s,
-                                            DEVICE_NATIVE_ENDIAN));
-    qdev_init_gpio_in(&dev->qdev, omap3_hsusb_musb_core_intr, __musb_irq_max);
+    memory_region_init_io(&s->iomem, &omap3_hsusb_otg_ops, s,
+                          "omap3_hsusb_otg", 0x1000);
+    sysbus_init_mmio(dev, &s->iomem);
+    qdev_init_gpio_in(&dev->qdev, omap3_hsusb_musb_core_intr, musb_irq_max);
     s->musb = musb_init(&dev->qdev, 0);
     vmstate_register(&dev->qdev, -1, &vmstate_omap3_hsusb_otg, s);
     return 0;
 }
 
-static SysBusDeviceInfo omap3_hsusb_otg_info = {
-    .init = omap3_hsusb_otg_init,
-    .qdev.name = "omap3_hsusb_otg",
-    .qdev.size = sizeof(OMAP3HSUSBOTGState),
-    .qdev.reset = omap3_hsusb_otg_reset,
+static void omap3_hsusb_otg_class_init(ObjectClass *klass, void *data)
+{
+    DeviceClass *dc = DEVICE_CLASS(klass);
+    SysBusDeviceClass *k = SYS_BUS_DEVICE_CLASS(klass);
+    k->init = omap3_hsusb_otg_init;
+    dc->reset = omap3_hsusb_otg_reset;
+}
+
+static TypeInfo omap3_hsusb_otg_info = {
+    .name = "omap3_hsusb_otg",
+    .parent = TYPE_SYS_BUS_DEVICE,
+    .instance_size = sizeof(OMAP3HSUSBOTGState),
+    .class_init = omap3_hsusb_otg_class_init,
 };
 
 typedef struct omap3_hsusb_host_s {
     SysBusDevice busdev;
+    MemoryRegion tll_iomem;
+    MemoryRegion host_iomem;
+    MemoryRegion ehci_iomem;
     qemu_irq ehci_irq;
     qemu_irq tll_irq;
     
@@ -310,7 +325,7 @@ static void omap3_hsusb_host_write(void *opaque, target_phys_addr_t addr,
     switch (addr) {
         case 0x00: /* UHH_REVISION */
         case 0x14: /* UHH_SYSSTATUS */
-            OMAP_RO_REGV(addr, value);
+            OMAP_RO_REG(addr);
             break;
         case 0x10: /* UHH_SYSCONFIG */
             s->uhh_sysconfig = value & 0x311d;
@@ -325,21 +340,25 @@ static void omap3_hsusb_host_write(void *opaque, target_phys_addr_t addr,
             s->uhh_debug_csr = value & 0xf00ff;
             break;
         default:
-            OMAP_BAD_REGV(addr, value);
+            OMAP_BAD_REG(addr);
             break;
     }
 }
 
-static CPUReadMemoryFunc *omap3_hsusb_host_readfn[] = {
-    omap_badwidth_read32,
-    omap_badwidth_read32,
-    omap3_hsusb_host_read,
-};
-
-static CPUWriteMemoryFunc *omap3_hsusb_host_writefn[] = {
-    omap_badwidth_write32,
-    omap_badwidth_write32,
-    omap3_hsusb_host_write,
+static const MemoryRegionOps omap3_hsusb_host_ops = {
+    .old_mmio = {
+        .read = {
+            omap_badwidth_read32,
+            omap_badwidth_read32,
+            omap3_hsusb_host_read,
+        },
+        .write = {
+            omap_badwidth_write32,
+            omap_badwidth_write32,
+            omap3_hsusb_host_write,
+        },
+    },
+    .endianness = DEVICE_NATIVE_ENDIAN,
 };
 
 static uint32_t omap3_hsusb_ehci_read(void *opaque, target_phys_addr_t addr)
@@ -369,16 +388,20 @@ static void omap3_hsusb_ehci_write(void *opaque, target_phys_addr_t addr,
      }
 }
 
-static CPUReadMemoryFunc *omap3_hsusb_ehci_readfn[] = {
-    omap_badwidth_read32,
-    omap_badwidth_read32,
-    omap3_hsusb_ehci_read,
-};
-
-static CPUWriteMemoryFunc *omap3_hsusb_ehci_writefn[] = {
-    omap_badwidth_write32,
-    omap_badwidth_write32,
-    omap3_hsusb_ehci_write,
+static const MemoryRegionOps omap3_hsusb_ehci_ops = {
+    .old_mmio = {
+        .read = {
+            omap_badwidth_read32,
+            omap_badwidth_read32,
+            omap3_hsusb_ehci_read,
+        },
+        .write = {
+            omap_badwidth_write32,
+            omap_badwidth_write32,
+            omap3_hsusb_ehci_write,
+        },
+    },
+    .endianness = DEVICE_NATIVE_ENDIAN,
 };
 
 static uint32_t omap3_hsusb_tll_read(void *opaque, target_phys_addr_t addr)
@@ -412,27 +435,31 @@ static void omap3_hsusb_tll_write(void *opaque, target_phys_addr_t addr,
     switch (addr) {
         case 0x00: /* USBTLL_REVISION */
         case 0x14: /* USBTLL_SYSSTATUS */
-            OMAP_RO_REGV(addr, value);
+            OMAP_RO_REG(addr);
             break;
         case 0x10: /* USBTLL_SYSCONFIG */
             s->tll_sysconfig = value & 0xFFFFFEE0;;
             break;
         default:
-            OMAP_BAD_REGV(addr, value);
+            OMAP_BAD_REG(addr);
             break;
     }
 }
 
-static CPUReadMemoryFunc *omap3_hsusb_tll_readfn[] = {
-    omap_badwidth_read32,
-    omap_badwidth_read32,
-    omap3_hsusb_tll_read,
-};
-
-static CPUWriteMemoryFunc *omap3_hsusb_tll_writefn[] = {
-    omap_badwidth_write32,
-    omap_badwidth_write32,
-    omap3_hsusb_tll_write,
+static const MemoryRegionOps omap3_hsusb_tll_ops = {
+    .old_mmio = {
+        .read = {
+            omap_badwidth_read32,
+            omap_badwidth_read32,
+            omap3_hsusb_tll_read,
+        },
+        .write = {
+            omap_badwidth_write32,
+            omap_badwidth_write32,
+            omap3_hsusb_tll_write,
+        },
+    },
+    .endianness = DEVICE_NATIVE_ENDIAN,
 };
 
 static int omap3_hsusb_host_init(SysBusDevice *dev)
@@ -440,34 +467,40 @@ static int omap3_hsusb_host_init(SysBusDevice *dev)
     OMAP3HSUSBHostState *s = FROM_SYSBUS(OMAP3HSUSBHostState, dev);
     sysbus_init_irq(dev, &s->ehci_irq);
     sysbus_init_irq(dev, &s->tll_irq);
-    sysbus_init_mmio(dev, 0x1000,
-                     cpu_register_io_memory(omap3_hsusb_tll_readfn,
-                                            omap3_hsusb_tll_writefn, s,
-                                            DEVICE_NATIVE_ENDIAN));
-    sysbus_init_mmio(dev, 0x400,
-                     cpu_register_io_memory(omap3_hsusb_host_readfn,
-                                            omap3_hsusb_host_writefn, s,
-                                            DEVICE_NATIVE_ENDIAN));
-    sysbus_init_mmio(dev, 0x400,
-                     cpu_register_io_memory(omap3_hsusb_ehci_readfn,
-                                            omap3_hsusb_ehci_writefn, s,
-                                            DEVICE_NATIVE_ENDIAN));
+
+    memory_region_init_io(&s->tll_iomem, &omap3_hsusb_tll_ops, s,
+                              "omap_usb.tll", 0x1000);
+    memory_region_init_io(&s->host_iomem, &omap3_hsusb_host_ops, s,
+                              "omap_usb.tll", 0x400);
+    memory_region_init_io(&s->ehci_iomem, &omap3_hsusb_ehci_ops, s,
+                              "omap_usb.tll", 0x400);
+    sysbus_init_mmio(dev, &s->tll_iomem);
+    sysbus_init_mmio(dev, &s->host_iomem);
+    sysbus_init_mmio(dev, &s->ehci_iomem);
     /* OHCI is instantiated by omap3.c */
     vmstate_register(&dev->qdev, -1, &vmstate_omap3_hsusb_host, s);
     return 0;
 }
 
-static SysBusDeviceInfo omap3_hsusb_host_info = {
-    .init = omap3_hsusb_host_init,
-    .qdev.name = "omap3_hsusb_host",
-    .qdev.size = sizeof(OMAP3HSUSBHostState),
-    .qdev.reset = omap3_hsusb_host_reset,
-};
-
-static void omap3_hsusb_register_devices(void)
+static void omap3_hsusb_host_class_init(ObjectClass *klass, void *data)
 {
-    sysbus_register_withprop(&omap3_hsusb_otg_info);
-    sysbus_register_withprop(&omap3_hsusb_host_info);
+    DeviceClass *dc = DEVICE_CLASS(klass);
+    SysBusDeviceClass *k = SYS_BUS_DEVICE_CLASS(klass);
+    k->init = omap3_hsusb_host_init;
+    dc->reset = omap3_hsusb_host_reset;
 }
 
-device_init(omap3_hsusb_register_devices)
+static TypeInfo omap3_hsusb_host_info = {
+    .name = "omap3_hsusb_host",
+    .parent = TYPE_SYS_BUS_DEVICE,
+    .instance_size = sizeof(OMAP3HSUSBHostState),
+    .class_init = omap3_hsusb_host_class_init,
+};
+
+static void omap3_hsusb_register_types(void)
+{
+    type_register_static(&omap3_hsusb_otg_info);
+    type_register_static(&omap3_hsusb_host_info);
+}
+
+type_init(omap3_hsusb_register_types)
